/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Pages 7 & 8 (Tables 1.1, 1.2 & 1.3)
**
*/

new;
library rpb,pgraph;

cls;

let mu =    0.05 0.06 0.08 0.06;
let sigma = 0.15 0.20 0.25 0.30;
n = rows(sigma);
let rho =   1.00 
            0.10   1.00
            0.40   0.70   1.00
            0.50   0.40   0.80 1.00;
rho = xpnd(rho);
call invpd(rho);

cov = rho .* sigma .* sigma';
A = ones(1,n); B = 1;

Bounds = (-10~10) .* ones(n,1);

TexFile = "chap1-1-1-3.tex";
mask = 1;

let phi = 10 5 2 1 0.50 0.20;
nPhi = rows(phi);
gamma_ = 1 ./ phi;
gamma_[1] = 0; 
{x,mu_x,sigma_x,gamma_x,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,gamma_,0);  
{x_min,sigma_min,retcode} = Compute_MV_Portfolio(cov,A,B,0,0,Bounds);
phi_x = 1/gamma_x;
print ftosa(100*(x_min~x),"%lf",3,2);
print "------------------------------------------------------------------------------------";

Results = phi_x'|100*(x|mu_x'|sigma_x');
fmt = ((0 $+ "%lf")~6~2) .* ones(cols(Results),1);
call Tabular(TeXFile,Results,mask,fmt,0,0,0,0,1);
print ftosa(Results,"%lf",3,2);
print "------------------------------------------------------------------------------------";

let mu_target = 0.05 0.06 0.07 0.08 0.09;
{x,mu_x,sigma_x,gamma_x,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,mu_target,1);  
phi_x = 1/gamma_x;
Results = 100*(mu_x'|x|sigma_x')|phi_x';
fmt = ((0 $+ "%lf")~6~2) .* ones(cols(Results),1);
call Tabular(TeXFile,Results,mask,fmt,0,0,0,0,0);
print ftosa(Results,"%lf",3,2);
print "------------------------------------------------------------------------------------";

let sigma_target = 0.15 0.20 0.25 0.30 0.35;
{x,mu_x,sigma_x,gamma_x,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,sigma_target,2);  
phi_x = 1/gamma_x;
Results = 100*(sigma_x'|x|mu_x')|phi_x';
fmt = ((0 $+ "%lf")~6~2) .* ones(cols(Results),1);
call Tabular(TeXFile,Results,mask,fmt,0,0,0,0,0);
print ftosa(Results,"%lf",3,2);
print "------------------------------------------------------------------------------------";
