/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 91 (Monte Carlo simulation)
**
*/

new;
library rpb,pgraph;

cls;

rndseed 123;

theta = 2;

mu1 = 0.10;
sigma1 = 0.20;
n1 = 6;
w1 = 0.50;

mu2 = 0.10;
sigma2 = 0.25;
n2 = 4;
w2 = 0.50;

w = w1|w2;

alpha = 0.90;
nS = 5000;
nR = 2000;
indx_alpha = nS * alpha;

h = 2.5/100;
h = h * nS;
indx = seqa(indx_alpha-h,1,2*h+1);
p = zeros(nS,1);
p[indx] = ones(2*h+1,1)/(2*h+1);

SD = zeros(nR,1);
RC_SD = zeros(2,nR);
VaR = zeros(nR,1);
RC_VaR = zeros(2,nR);
VaR_w = zeros(nR,1);
RC_VaR_w = zeros(2,nR);
ES = zeros(nR,1);
RC_ES = zeros(2,nR);
ES_w = zeros(nR,1);
RC_ES_w = zeros(2,nR);

i = 1;
do until i > nR;
  {u1,u2} = rndCopulaClayton(theta,ns);

  R1 = mu1 + sigma1 * cdfti(u1,n1);
  R2 = mu2 + sigma2 * cdfti(u2,n2);

  cov = vcx(R1~R2);
  SD[i] = sqrt(w'cov*w);
  RC_SD[.,i] = w .* (cov*w)/sqrt(w'cov*w);

  L1 = -w1 * R1;
  L2 = -w2 * R2;

  L = L1~L2;

  {VaR[i],RC_VaR[.,i],ES[i],RC_ES[.,i]} = Compute_Empirical_RC(L,alpha);
  {VaR_w[i],RC_VaR_w[.,i],ES_w[i],RC_ES_w[.,i]} = Compute_Empirical_RC(L,p);

  i = i + 1;
endo;

RC_SD = RC_SD';
RC_VaR = RC_VaR';
RC_ES = RC_ES';
RC_VaR_w = RC_VaR_w';

results = ((meanc(SD)|stdc(SD))                     |
          (meanc(RC_SD[.,1])|stdc(RC_SD[.,1]))     |
          (meanc(RC_SD[.,2])|stdc(RC_SD[.,2]))     ) ~
          ((meanc(VaR)|stdc(VaR))                     |
          (meanc(RC_VaR[.,1])|stdc(RC_VaR[.,1]))     |
          (meanc(RC_VaR[.,2])|stdc(RC_VaR[.,2]))     )~
          ((meanc(ES)|stdc(ES))                     |
          (meanc(RC_ES[.,1])|stdc(RC_ES[.,1]))     |
          (meanc(RC_ES[.,2])|stdc(RC_ES[.,2]))     )~
          ((meanc(VaR_w)|stdc(VaR_w))                     |
          (meanc(RC_VaR_w[.,1])|stdc(RC_VaR_w[.,1]))     |
          (meanc(RC_VaR_w[.,2])|stdc(RC_VaR_w[.,2]))     );
print ftosa(100*results,"%lf",3,2);
print "--------------------------------------------------------------------";

results = results[3 5,.]./results[1,.];
print ftosa(100*results,"%lf",3,2);