/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 144 (Solves the duplication invariance property)
**
*/

new;

#define use_co 1

#ifdef use_co
  library rpb,pgraph,co;
#else
  library rpb,pgraph;
#endif

cls;

let sigma[4,1] = 0.20 0.10 0.15 0.30;
m = rows(sigma);
let rho = 1.0
          0.5 1.0
          0.25 0.15 1.0
          0.10 0.30 0.40 1.0;
rho = xpnd(rho);
cov1 = rho .* sigma .* sigma';

n = m + 1; 
sigma = sigma | sigma[m];
rho = (rho~rho[.,m])|(rho[m,.]~1.0);
cov2 = rho .* sigma .* sigma';

let b1 = 0.25 0.25 0.30 0.20;
{x1,vol1,mr1,rc1,prc1} = Compute_RB_Portfolio(0,cov1,b1);

let b2 = 0.20 0.20 0.20 0.20 0.20;
{x2,vol2,mr2,rc2,prc2} = Compute_RB_Portfolio(0,cov2,b2);

b3 = b1[1:m-1] | b1[m]/2 | b1[m]/2;
{x3,vol3,mr3,rc3,prc3} = Compute_RB_Portfolio(0,cov2,b3);

A = eye(m)|(zeros(1,m-1)~1);
Omega = cov1;
D = zeros(n,n);
cov4 = A*omega*A' + D;

b4 = b1;
indx = seqa(1,1,m);

_rpb_factor_bounds = -1;

sv = x3;

{x,y,f_mr,f_rc,f_prc} = Compute_RB_Factor(sv,cov4,A,Omega,D,b4,indx,1); 
{vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov4);

cls;

print "------------------------------------------------------";

print ftosa(100*vol1,"%lf",3,2);
print ftosa(100*(x1~mr1~rc1~prc1),"%lf",3,2);

print ftosa(100*vol2,"%lf",3,2);
print ftosa(100*(x2~mr2~rc2~prc2),"%lf",3,2);

print ftosa(100*vol3,"%lf",3,2);
print ftosa(100*(x3~mr3~rc3~prc3),"%lf",3,2);

print "------------------------------------------------------";

print ftos(100*vol,"vol = %lf",3,2);
print ftosa(100*(y~f_mr~f_rc~f_prc),"%lf",3,2);

print ftos(100*vol,"vol = %lf",3,2);
print ftosa(100*(x~mr~rc~prc),"%lf",3,2);