/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 168 (Table 3.4)
**
*/

new;
library rpb,pgraph;

cls;

rndseed 123;

let beta =  0.9 0.8 1.2 0.7 1.3;
sigma_m = 0.25;
sigma_m_sqr = sigma_m^2;
let sigma_tilde = 0.04 0.12 0.05 0.08 0.05;

n = rows(beta);
D = diagrv(zeros(n,n),sigma_tilde^2);
Sigma = beta*beta'*sigma_m_sqr + D;

rho = corrvc(Sigma);
vol = sqrt(diag(Sigma));
print ftosa(100*(vol~rho),"%lf",3,2);
print "-----------------------------------------------------------------------";

invSigma = invpd(Sigma);
beta_tilde = beta ./ (sigma_tilde^2);
kappa = beta'beta_tilde;
iMatrix = invpd(D) - sigma_m_sqr/(1+sigma_m_sqr*kappa)*beta_tilde*beta_tilde';
e = ones(n,1);

print ftosa(invSigma,"%lf",3,3);
print ftosa(iMatrix,"%lf",3,3);
print "-----------------------------------------------------------------------";

{x1,vol1,retcode} = Compute_MV_Portfolio(Sigma,ones(1,n),1,0,0,-5~5);
vol2 = 1/sqrt(e'*invSigma*e);
x2 = (vol2^2)*iMatrix*e;
c = sigma_m_sqr/(1+sigma_m_sqr*kappa)*sumc(beta_tilde);
beta_star = (1 + sigma_m_sqr*kappa)/(sigma_m_sqr * sumc(beta_tilde));
x3 = (vol2^2) ./ (sigma_tilde^2) .* (1 - beta/beta_star);
vol3 = sqrt(x3'Sigma*x3);

print;
print ftosa(kappa,"kappa = %lf",3,4);
print ftosa(beta_star,"beta* = %lf",3,4);
print ftosa(100*(x1~x2~x3),"%lf",3,3);
print;
print ftosa(100*(vol1~vol2~vol3),"%lf",3,3);

print "-----------------------------------------------------------------------";

{x4,vol4,retcode} = Compute_MV_Portfolio(Sigma,ones(1,n),1,0,0,0~1);
e = x4 .> 0;
kappa = sumc( e .* beta .* beta_tilde);
beta_star = (1 + sigma_m_sqr*kappa)/(sigma_m_sqr * sumc(e .* beta_tilde));
x5 = (vol4^2) ./ (sigma_tilde^2) .* (1 - beta/beta_star);
x5 = e .* x5;
vol5 = sqrt(x5'Sigma*x5);

print ftosa(kappa,"kappa = %lf",3,4);
print ftosa(beta_star,"beta* = %lf",3,4);
print ftosa(100*(x1~x4~x5),"%lf",3,3);
print;
print ftosa(100*(vol1~vol4~vol5),"%lf",3,3);
print "-----------------------------------------------------------------------";

results = beta~beta_tilde~100*x3~100*x5|
          (error(0)~error(0)~100*vol2~100*vol4);

print ftosa(results,"%lf",3,2);
