/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 185 (Figure 3.8)
**
*/

new;
library rpb,pgraph;

cls;

PathName = "data//";
datFileName = Pathname $+ "sx5e-info";
loads info = ^datFileName;
datFileName = Pathname $+ "sx5e-weights";
Composition = loadd(datFileName);
datFileName = Pathname $+ "sx5e-prices";
Prices = loadd(datFileName);
nStocks = cols(Prices) - 2;

Weights = Composition[.,2:cols(Composition)]/100;

Rebalancing_Dates = Composition[.,1];
Rebalancing_Dates[1] = 19920101;
nRD = rows(Rebalancing_Dates);

Dates = Prices[.,1];
Index = Prices[.,2];
Prices = Prices[.,3:2+nStocks];

EndDate = 20120928;

Lag_CovMatrix = 260;
FileName = itos("Results\\sx5e-Weights-basic-",Lag_CovMatrix);
loadm Computed_Weights = ^FileName;
FileName = itos("Results\\sx5e-RC-basic-",Lag_CovMatrix);
loadm Computed_RC = ^FileName;

nAW = submat(getorders(Computed_Weights),1,0);

q = seqa(1,1,50)/50;
Lorenz_W = zeros(50,nAW+1);
Lorenz_RC = zeros(50,nAW);

Begin_Iter = 13;
i = Begin_Iter;
do until i > nRD;

  tmp_Computed_Weights = array3d_get_y(Computed_Weights,i)';
  tmp_Index_Weights = Weights[i,.]';
  tmp_Weights = tmp_Computed_Weights~tmp_Index_Weights;
  indx = selif(seqa(1,1,nStocks),tmp_Index_Weights[.,1] ./= 0.0);
  tmp_Weights = tmp_Weights[indx,.];

  tmp_Computed_RC = array3d_get_y(Computed_RC,i)';
  tmp_RC = tmp_Computed_RC;
  tmp_RC = tmp_RC[indx,.];

  j = 1;
  do until j > nAW;
    tmp = tmp_Weights[.,j];
    tmp = rev(sortc(tmp,1));
    tmp = cumsumc(tmp);
    tmp = tmp/tmp[rows(tmp)];
    x = seqa(1,1,rows(tmp))/rows(tmp);
    y = fspline(csspline(x,tmp,1,1),q);
    Lorenz_W[.,j] = Lorenz_W[.,j] + y;

    tmp = tmp_RC[.,j];
    tmp = rev(sortc(tmp,1));
    tmp = cumsumc(tmp);
    x = seqa(1,1,rows(tmp))/rows(tmp);
    y = fspline(csspline(x,tmp,1,1),q);
    Lorenz_RC[.,j] = Lorenz_RC[.,j] + y;

    j = j + 1;
  endo; 

// test
    tmp = tmp_Weights[.,j];
    tmp = rev(sortc(tmp,1));
    tmp = cumsumc(tmp);
    x = seqa(1,1,rows(tmp))/rows(tmp);
    y = fspline(csspline(x,tmp,1,1),q);
    Lorenz_W[.,j] = Lorenz_W[.,j] + y;
 
  i = i + 1;
endo;

Lorenz_W = Lorenz_W/(nRD-Begin_Iter+1);
Lorenz_W = zeros(1,nAW+1) | Lorenz_W;

Lorenz_RC = Lorenz_RC/(nRD-Begin_Iter+1);
Lorenz_RC = zeros(1,nAW) | Lorenz_RC;

q = 0|q;

FileName = itos("Results\\sx5e-Weights-",Lag_CovMatrix);
loadm Computed_Weights = ^FileName;

nAW = 8;
maxWeights = zeros(nRD,nAW);
nAssets = zeros(nRD,nAW);
lower_bound = 0.00001;
allWeights = miss(zeros(nRD*50,nAW),0);

i = 1;
do until i > nAW;
  Weights = array3d_get_x(Computed_Weights,i)';
  m = miss(maxc(Weights),0);
  maxWeights[.,i] = m;
  nAssets[.,i] = sumr(Weights' .> lower_bound);
  Weights = vecr(Weights);
  Weights = selif(Weights, Weights .> lower_bound);
  allWeights[1:rows(Weights),i] = Weights;
  i = i + 1;
endo;

nAssets = miss(nAssets,0);

graphset;
  begwind;
  window(2,2,0);  
    _pdate = ""; _pnum = 2; _pframe = 0;
    _pltype = 1|3|6|6; _plwidth = 10; _pcolor = 5|1|2|4; 
    _plctrl = 0|0|0|10; _pstype = 0|0|0|8; _psymsiz = 5;
    ytics(0,100,20,2);
    xtics(0,100,20,2);
    _plegstr = "MV 10%\000MV 5%\000MDP 10%\000MDP 5%";
    _paxht = 0.25; _pnumht = 0.225; _ptitlht = 0.25;

  setwind(1);
    title("\216Weights");
    xy(100*q,100*Lorenz_W[.,5 7 6 8]);

  setwind(2);
    title("\216Risk contributions");
    _plegctl = {2 5 5.5 0.9};
    xy(100*q,100*Lorenz_RC[.,5 7 6 8]);

  _plegctl = 0; _plctrl = 0;

  setwind(3);
    title("\216Weights");
    _pboxctl = { .25 0 1 }; _plwidth = 10;
    xtics(0,4,1,0);
    let xlab = "MV 10%" "MV 5%" "MDP 10%" "MDP 5%";
    asclabel(xlab,0);
    ytics(0,15,5,2);
    x = allWeights[.,5 7 6 8];
    call box(0,100*x);

  setwind(4);
    title("\216Number of invested stocks");
    ytics(0,40,10,2);
    x = packr(nAssets[.,5 7 6 8]);
    call box(0,x);

    graphprt("-c=1 -cf=chap3-3-2-3.eps");
  endwind;

cls;

x = packr(nAssets[.,5 7 6 8]);
print ftosa(minc(x)~median(x)~maxc(x),"%lf",3,0);


