/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 206 (Table 4.5)
**
*/

new;
library rpb,pgraph;

cls;

_Bond_Continuous_Time = 1;
_bond_window = 2;

loadm maturity = data\zc-tau;

datFileName = "data\\zc-ir-2009.dat";
data = loadd(datFileName);
data = selif(data,(data[.,1] .>= 20100630) .and (data[.,1] .<= 20120630));
Dates = data[.,1];
nDates = rows(Dates);
R = data[.,2:cols(data)]/100;
tau = seqa(1,1,5);
indx = indnv(tau,Maturity);
R = R[.,indx];

rndseed 123456;
t = seqa(0,1/260,nDates);
Sp = simulate_GBM(1,0.50,0.40,t,1)/100;

Rt = R[rows(R),.]';

Nominal = 1e6;
Coupon = 5;
Notional = 100;
Maturity = 5;
Ct = Coupon * ones(Maturity,1);
Ct[Maturity] = Ct[Maturity] + Notional;
t = seqa(1,1,Maturity);

P0 = Bond_Price(t,Ct,Rt);
nB = Nominal/P0;
nB = trunc(nB);

dR = R - lag1(R);
{gVaR1,gVaR2,hVaR1,hVaR2,hVaR3,ES,results} = Bond_VaR(t,Ct,Rt,dR,0.99,1/260);
print ftosa(nB*(gVaR1~gVaR2~hVaR1~hVaR2~hVaR3~ES),"%lf",5,1);
print "--------------------------------------------------------------------------------------------------------";

{RC_gVaR1,RC_gVaR2,RC_hVaR1,RC_hVaR2,RC_hVaR3,RC_ES} = Bond_VaR_RC(t,Ct,Rt,dR,0.99,1/260);
print ftosa(nB*(RC_gVaR1~RC_gVaR2~RC_hVaR1~RC_hVaR2~RC_hVaR3~RC_ES),"%lf",5,1);
print;
print ftosa(nB*sumc(RC_gVaR1~RC_gVaR2~RC_hVaR1~RC_hVaR2~RC_hVaR3~RC_ES)',"%lf",5,1);
print "--------------------------------------------------------------------------------------------------------";

{RC_gVaR1,RC_gVaR2,RC_hVaR1,RC_hVaR2,RC_hVaR3,RC_ES} = Bond_Portfolio_VaR_RC(nB,t,Ct,Rt,dR,0.99,1/260,1);

print ftosa((RC_gVaR1~RC_gVaR2~RC_hVaR1~RC_hVaR2~RC_hVaR3~RC_ES),"%lf",5,1);
print;
print ftosa(sumc(RC_gVaR1~RC_gVaR2~RC_hVaR1~RC_hVaR2~RC_hVaR3~RC_ES)',"%lf",5,1);
print "--------------------------------------------------------------------------------------------------------";

{RC_gVaR1,RC_gVaR2,RC_hVaR1,RC_hVaR2,RC_hVaR3,RC_ES} = Bond_Portfolio_VaR_RC(nB,t,Ct,Rt,dR,0.99,1/260,2);

print ftosa((RC_gVaR1~RC_gVaR2~RC_hVaR1~RC_hVaR2~RC_hVaR3~RC_ES),"%lf",5,1);
print;
print ftosa(sumc(RC_gVaR1~RC_gVaR2~RC_hVaR1~RC_hVaR2~RC_hVaR3~RC_ES)',"%lf",5,1);
print "--------------------------------------------------------------------------------------------------------";
