/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 208 (Table 4.6)
**
*/

new;
library rpb,pgraph;

cls;

_Bond_Continuous_Time = 1;
_bond_window = 2;

loadm maturity = data\zc-tau;

datFileName = "data\\zc-ir-2009.dat";
data = loadd(datFileName);
data = selif(data,(data[.,1] .>= 20100630) .and (data[.,1] .<= 20120630));
Dates = data[.,1];
nDates = rows(Dates);
R = data[.,2:cols(data)]/100;
tau = seqa(1,1,5);
indx = indnv(tau,Maturity);
R1 = R[.,indx];
Rt1 = R1[rows(R),.]';

tau = seqa(0.5,1,5);
indx = indnv(tau,Maturity);
R2 = R[.,indx];
Rt2 = R2[rows(R),.]';

tau = seqa(1,1,2);
indx = indnv(tau,Maturity);
R3 = R[.,indx];
Rt3 = R3[rows(R),.]';

rndseed 123456;
t = seqa(0,1/260,nDates);
Sp = simulate_GBM(1,0.50,0.40,t,1)/100;

tau = seqa(0.5,0.5,10);
indx = indnv(tau,Maturity);
R = R[.,indx];
Rt = R[rows(R),.]';

// Bond #1

Nominal = 4e5;
Coupon = 5;
Notional = 100;
Maturity = 5;
Ct = Coupon * ones(Maturity,1);
Ct[Maturity] = Ct[Maturity] + Notional;
t = seqa(1,1,Maturity);

P0 = Bond_Price(t,Ct,Rt1);
nB = trunc(Nominal/P0);

nB1 = nB; t1 = t; Ct1 = Ct; P1 = P0;

// Bond #2

Nominal = 4e5;
Coupon = 2;
Notional = 120;
Maturity = 5;
Ct = Coupon * ones(Maturity,1);
Ct[Maturity] = Ct[Maturity] + Notional;
t = seqa(0.5,1,Maturity);

P0 = Bond_Price(t,Ct,Rt2);
nB = trunc(Nominal/P0);

nB2 = nB; t2 = t; Ct2 = Ct; P2 = P0;

// Bond #3

Nominal = 2e5;
Coupon = 3;
Notional = 100;
Maturity = 2;
Ct = Coupon * ones(Maturity,1);
Ct[Maturity] = Ct[Maturity] + Notional;
t = seqa(1,1,Maturity);

P0 = Bond_Price(t,Ct,Rt3);
nB = trunc(Nominal/P0);

nB3 = nB; t3 = t; Ct3 = Ct; P3 = P0;

dR = R - lag1(R);

{t,Ct} = Bond_Generate_Portfolio(t1,Ct1,t2,Ct2);
{t,Ct} = Bond_Generate_Portfolio(t,Ct,t3,Ct3);

nB = nB1 | nB2 | nB3;
P = P1 | P2 | P3;
w =  (nB .* P)/sumc(nB .* P);
print ftosa(nB~P~100*w,"%lf",3,2);
print "--------------------------------------------------------------------------------------------------------";

{gVaR1,gVaR2,hVaR1,hVaR2,hVaR3,ES} = Bond_Portfolio_VaR(nB,t,Ct,Rt,dR,0.99,1/260,1);
print ftosa((gVaR1~gVaR2~hVaR1~hVaR2~hVaR3~ES),"%lf",5,1);
print "--------------------------------------------------------------------------------------------------------";

{gVaR1,gVaR2,hVaR1,hVaR2,hVaR3,ES} = Bond_Portfolio_VaR(nB,t,Ct,Rt,dR,0.99,1/260,2);
print ftosa((gVaR1~gVaR2~hVaR1~hVaR2~hVaR3~ES),"%lf",5,1);
print "--------------------------------------------------------------------------------------------------------";

{RC_gVaR1,RC_gVaR2,RC_hVaR1,RC_hVaR2,RC_hVaR3,RC_ES} = Bond_Portfolio_VaR_RC(nB,t,Ct,Rt,dR,0.99,1/260,1);

print ftosa((RC_gVaR1~RC_gVaR2~RC_hVaR1~RC_hVaR2~RC_hVaR3~RC_ES),"%lf",5,1);
print;
print ftosa(sumc(RC_gVaR1~RC_gVaR2~RC_hVaR1~RC_hVaR2~RC_hVaR3~RC_ES)',"%lf",5,1);
print "--------------------------------------------------------------------------------------------------------";

{RC_gVaR1,RC_gVaR2,RC_hVaR1,RC_hVaR2,RC_hVaR3,RC_ES} = Bond_Portfolio_VaR_RC(nB,t,Ct,Rt,dR,0.99,1/260,2);

print ftosa((RC_gVaR1~RC_gVaR2~RC_hVaR1~RC_hVaR2~RC_hVaR3~RC_ES),"%lf",5,1);
print;
print ftosa(sumc(RC_gVaR1~RC_gVaR2~RC_hVaR1~RC_hVaR2~RC_hVaR3~RC_ES)',"%lf",5,1);
print "--------------------------------------------------------------------------------------------------------";

RC = (RC_gVaR1~RC_gVaR2~RC_hVaR1~RC_hVaR2~RC_hVaR3~RC_ES) ./ sumc(RC_gVaR1~RC_gVaR2~RC_hVaR1~RC_hVaR2~RC_hVaR3~RC_ES)';

print ftosa(100*RC,"%lf",5,1);
print "--------------------------------------------------------------------------------------------------------";

