/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 252 (Tables 5.1 & 5.2)
**
*/

new;
library rpb,pgraph;

cls;

datFileName = "data\\commodities";

data = loadd(datFileName);
vnames = "" $+ getname(datFileName);
loads strData = ^datFileName;

let string Selected_Commodities = "CL" "CO" "NG" "HO" "W" "C" "S" "CT" "HG" "GC" "SI" "PL";

nCommodities = rows(Selected_Commodities);

Dates = Data[.,1];
indx = indsav(Selected_Commodities,vnames);
Data = Data[.,indx];
Data = Fill_Miss(Data,4);

BeginDate = 19900101;
Basket = selif(Data,Dates .>= BeginDate);
Dates = selif(Dates,Dates .>= BeginDate);

vol = zeros(nCommodities+1,6);

i = 1;
do until i > nCommodities;
  Rdt = Basket[.,i]./lag1(basket[.,i]) - 1;
  Rdt = packr(Rdt);
  Vol[i,1] = sqrt(260) * stdc(Rdt);
  
  sigma = Rolling_Volatility(packr2(Dates,Basket[.,i]),260);
  
  sigma = packr(sigma);
  vol[i,2:6] = minc(sigma) ~ quantile(sigma,0.10) ~ quantile(sigma,0.50) ~ quantile(sigma,0.90) ~  maxc(sigma);  

  i = i + 1;
endo;

vol[nCommodities+1,1] = meanc(vol[1:nCommodities,1]);

mu = zeros(nCommodities+1,6);

i = 1;
do until i > nCommodities;
  {indx_Begin,indx_Last} = indn_nomiss(Basket[.,i]);
  dt = etdays2(dates[indx_Begin],dates[indx_last])/365;
  Rt = (Basket[indx_Begin,i]./Basket[indx_Last,i])^(1/dt) - 1;
  mu[i,1] = Rt;
  
  Rt = Rolling_Return(Dates,Basket[.,i],260);
  Rt = packr(Rt);
  mu[i,2:6] = minc(Rt) ~ quantile(Rt,0.10) ~ quantile(Rt,0.50) ~ quantile(Rt,0.90) ~  maxc(Rt);  
  i = i + 1;
endo;

mu[nCommodities+1,1] = meanc(mu[1:nCommodities,1]);

str = Selected_Commodities $| "Mean";
print (str $~ ftosa(100*mu,"%lf",3,1));
print (str $~ ftosa(100*vol,"%lf",3,1));
