/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Pages 258 & 262 (Tests the convergence of ERC optimization problems)
**
*/

new;
library rpb,pgraph;

cls;

datPathName = "Data\\";
datFileName = datPathName $+ "djcs-hf";

resPathName = "Results\\DJCS-3F";

Data = loadd(datFileName);
vnames = "" $+ getname(datFileName);

let string Selected_Names[10,1] = CA SB EM EMN ED FIA GM LSE MF MS;

indx = indsav(Selected_Names,vnames);
nAssets = rows(indx);

Dates = Data[.,1];
nDates = rows(Dates);
t = seqa(2004 + 9/12,1/12,nDates);
Index_NAV = Data[.,2];
SubIndex_NAV = Data[.,indx];

R_SubIndex = SubIndex_NAV ./ lag1(SubIndex_NAV) - 1.0;
nLags = 24;

let x0 = 3 1 6 4 22 5 14 24 6 13;
x0 = x0 / sumc(x0);
index_weights = x0' .* ones(nDates,1);

let indx_Factors = 1 2 3;
let b_Factors = 1 1 1;
b_Factors = b_Factors ./ sumc(b_Factors);
nFactors = rows(indx_Factors);

loadm path = ^resPathName Asset_Weights;
loadm path = ^resPathName Asset_RC;
loadm path = ^resPathName Factor_RC;
loadm path = ^resPathName FRC_Index;
loadm path = ^resPathName test_retcode;
loadm path = ^resPathName test_fmin;

Index_Asset = zeros(nDates,2);
Index_Strategy = zeros(nDates,2);

INDEX_ERC = 2;

// Asset-Weighted

Weights = arraytomat(Asset_Weights[1,.,.]);
{Begin_Date,End_Date} = indn_nomiss(Weights[.,1]);
Weights = missrv(weights,index_weights);
R = sumr(Weights .* R_SubIndex);
R[1] = 0;
Index_Asset[.,1] = 100*cumprodc(1 +  R);

// ERC-Weighted

Asset_Weights = arraytomat(Asset_Weights[Index_ERC,.,.]);
Asset_RC = arraytomat(Asset_RC[Index_ERC,.,.]);
Factor_RC = arraytomat(Factor_RC[Index_ERC,.,.]);
FRC_Index = arraytomat(FRC_Index[Index_ERC,.,.]);
test_retcode = test_retcode[.,Index_ERC];
test_fmin = test_fmin[.,Index_ERC];

Weights = Asset_Weights;
{Begin_Date,End_Date} = indn_nomiss(Weights[.,1]);
Weights = missrv(weights,index_weights);
R = sumr(Weights .* R_SubIndex);
R[1] = 0;
Index_Asset[.,2] = 100*cumprodc(1 +  R);

indx_t = seqa(Begin_Date[1]-1,1,End_Date[1]-Begin_Date[1]+2);

Index_Asset = 100*Index_Asset./Index_Asset[indx_t[1],.];

Index_Strategy = Index_Asset;
Index_Strategy_Smoothed = Index_Strategy;

iter = 1;
do until iter > 2;
  c = csspline(t[indx_t],Index_Strategy[indx_t,iter],1,1);
  Index_Strategy_Smoothed[indx_t,iter] = fspline(c,t[indx_t]);
  iter = iter + 1;
endo;

graphset;
  begwind;
  window(2,3,0);
  _pdate = ""; _pnum = 2; _pframe = 0;
  _plwidth = 10; _pltype = 6|3; _pcolor = 1|4;
  _paxht = 0.20; _pnumht = 0.20; _ptitlht = 0.275;

  _pboxctl = {.25 0 3 10 90};
  _pcolor = seqa(1,1,9)|12;

  ytics(0,50,10,2);

  setwind(1);   
    title("\216Weights");  
    call box(0,100*Asset_Weights);

  ytics(0,100,25,2);

  setwind(2);     
    title("\216Asset RC");  
    call box(0,100*Asset_RC);

  setwind(3);     
    title("\216Factor RC");  
    call box(0,100*Factor_RC);

  setwind(4);
    title("\216Performance");  
    xtics(2006,2013,1,2);
    ytics(80,120,5,2);
    xy(t[indx_t],Index_Strategy_Smoothed[indx_t,.]);

  _plctrl = -1; _pstype = 8;
  setwind(5);
    title("\216retcode");  
    ytics(0,3,1,2);
    xy(t[indx_t],test_retcode[indx_t,.]);

  _plctrl = 0;
  setwind(6);
    title("\216fmin");  
    ytics(0,error(0),1,2);
    xy(t[indx_t],test_fmin[indx_t,.]);
  
  endwind;
