/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 268 (Tables 5.12 & 5.13)
**
*/

new;
library rpb,pgraph;

cls;

let sigma = 0.20 0.15 0.25 0.20;
let rho   = 1.00
            0.20 1.00
            0.10 0.15 1.00
            0.20 0.20 0.50 1.00;
rho = xpnd(rho);
call invpd(rho);
cov = rho .* sigma .* sigma';
n = rows(cov);

alpha = seqa(0,1/100,101);
nAlpha = rows(alpha);
tau = zeros(nAlpha,1);

let x0 = 0.20 0.40 0.05 0.35; 
{vol0,mr0,rc0,pc0} = Compute_Risk_Contribution(x0,cov);
b0 = pc0;

b = ones(n,1)/n;
{x1,vol1,mr1,rc1,pc1} = Compute_RB_Portfolio(x0,cov,b);
tau1 = sumc(abs(x1-x0));

i = 1;
do until i > nAlpha;
  b1 = alpha[i]*b + (1-alpha[i]) * b0;
  {x,vol,mr,rc,pc} = Compute_RB_Portfolio(x0,cov,b1);
  tau[i] = sumc(abs(x-x0));
  i = i + 1;
endo;

tau_max = 0.20;

proc objective(alpha);
  local b_x,x,vol,mr,rc,pc,tau;
  b_x = alpha*b + (1-alpha) * b0;
  {x,vol,mr,rc,pc} = Compute_RB_Portfolio(x0,cov,b_x);
  tau = sumc(abs(x-x0));
  retp(tau - tau_max);
endp;

alpha_b = Bisection(&objective,0,1);
b_x = alpha_b*b + (1-alpha_b) * b0;
{x,vol,mr,rc,pc} = Compute_RB_Portfolio(x0,cov,b_x);
tau = sumc(abs(x-x0));

cls;

results = (x0~mr0~rc0~b0)|(error(0)~error(0)~vol0~error(0));

print ftosa(100*results,"%lf",3,2);
print ftos(0,"tau = %lf",3,2);
print "---------------------------------------------------------------------------";

results = (x1~mr1~rc1~b1)|(error(0)~error(0)~vol1~error(0));

print ftosa(100*results,"%lf",3,2);
print ftos(100*tau1,"tau = %lf",3,2);
print "---------------------------------------------------------------------------";

results = (x~mr~rc~pc)|(error(0)~error(0)~vol~error(0));

print ftosa(100*results,"%lf",3,2);
print ftos(100*tau,"tau = %lf",3,2);
print ftos(100*alpha_b,"alpha(b) = %lf",3,2);
print "---------------------------------------------------------------------------";

alpha_x = _min_(tau_max/tau1,1.0);
x4 = alpha_x * x1 + (1-alpha_x) * x0;
tau4 = sumc(abs(x4-x0));

{vol4,mr4,rc4,pc4} =  Compute_Risk_Contribution(x4,cov);
results = (x4~mr4~rc4~pc4)|(error(0)~error(0)~vol4~error(0));

print ftosa(100*results,"%lf",3,2);
print ftos(100*tau4,"tau = %lf",3,2);
print ftos(100*alpha_x,"alpha(x) = %lf",3,2);
print "---------------------------------------------------------------------------";



